import array
import datetime
import logging
import re

import requests

from .const import commands, dict_channel_list, urls
from pyskycontrol import Helper

_LOGGER = logging.getLogger(__name__)


class SkyDevice:
    """Sky Control Class"""

    def __init__(
        self,
        host: str,
        ctrl_url: str,

    ) -> None:
        """Create a new SmartDevice instance.
        :param str host: host name or ip address on which the device listens.
        :param str ctrl_url: control url to control th box.
        """

        self.host = host
        self.ctrl_url = ctrl_url

    def get_status(self, host, ctrl_url):
        """Send soap command to the Sky Q box to get status"""
        test = Helper.request(request_type="GET",
                              response_type="JSON",
                              target=host,
                              request_url=urls["info_url"])

        if test["activeStandby"] != True:
            command = "GetTransportInfo"
            response = Helper.soap_request(command, ctrl_url)
            return response.find('CurrentTransportState').string
        else:
            response = 'STOPPED'
            return response

    def channel_epg(self, host):
        """Get the Sky EPG"""
        channel_list = {}
        request = Helper.request(request_type="GET",
                                 response_type="JSON",
                                 target=host,
                                 request_url=urls["channel_epg"])
        channels = request['services']
        for channel in channels:
            if channel["t"] not in channel_list:
                channel_list.update({channel["t"]: channel})
        channel_list.update({'Recording': {'sid': '0', 't': 'Recording'}})
        dict_channel_list = channel_list
        return channel_list

    def current_program(self, host, ctrl_url):
        """Get the current channel"""
        data = {}
        ps = None
        prog_type = None
        ch = None
        channel = None
        desc = None
        command = "GetMediaInfo"

        response = Helper.soap_request(command, ctrl_url)
        soap_result = response.find('CurrentURI').string

        if soap_result is None:
            data.update({'pvr_sid': 'off'})
            return data
        elif 'xsi' in soap_result:
            prog_type = 'Channel'
            ps = str(int(re.split('\\bxsi://\\b', soap_result)[-1], 16))
            data.update({'pvr_sid': ps})
            data.update({'type': prog_type})
        elif 'pvr' in soap_result:
            prog_type = 'Recording'
            ps = "P" + re.split('\\bfile://pvr/\\b', soap_result)[-1].lower()
            data.update({'pvr_sid': ps})
            data.update({'type': prog_type})

        if prog_type == 'Channel':
            d = datetime.datetime.now()
            d_date = d.strftime('%Y%m%d')
            response = Helper.request(request_type='GET',
                                      response_type='JSON',
                                      target="N/A",
                                      request_url=urls["chn_list"].format(d_date, ps))
            p_list = response["schedule"][0]["events"]
            current_epoch = Helper.epochtime(
                datetime.datetime.now().strftime('%d.%m.%Y %H:%M:%S'),
                None, 'to_epoch')
            for prog in p_list:
                if prog["st"] + prog["d"] > current_epoch:
                    ch = prog
                    break

            for cha in dict_channel_list:
                if ps == dict_channel_list[cha]["sid"]:
                    channel = dict_channel_list[cha]['t']
                    data.update({"channel": channel})
                    break

        elif prog_type == 'Recording':
            response = Helper.request(request_type='GET',
                                      response_type='JSON',
                                      target=host,
                                      request_url=urls["rec_list"])
            p_list = response["pvrItems"]
            for rec in p_list:
                if rec["pvrid"] == ps:
                    ch = rec
                    if 'cn' in ch:
                        channel = ch['cn']
                        data.update({"channel": channel})
                    if 'osid' in ch:
                        ps = ch['osid']
                    break

        if ch is not None:
            if 't' in ch:
                title = ch['t']
                data.update({"title": title})
            if 'sy' in ch:
                desc = ch['sy']
                if ':' in desc:
                    s_t = desc.split(":")
                    series_title = s_t[0]
                    data.update({'series_title': series_title})
            if 'seasonnumber' in ch and 'episodenumber' in ch:
                season = ch['seasonnumber']
                episode = ch['episodenumber']
                data.update({"season": season})
                data.update({"episode": episode})
                source_type = 'TV'
                data.update({"source_type": source_type})
            else:
                source_type = 'Video'
                data.update({"source_type": source_type})
            if 'programmeuuid' in ch:
                programmeuuid = ch['programmeuuid']
                imageurl = urls['metadata_url'].format(
                    str(programmeuuid), str(ps))
                data.update({"imageurl": imageurl})

        _LOGGER.debug(data)
        return data

        def remote(self, host, command):
            """Send a button press to the sky box."""



import math
import socket
import time

import requests
from bs4 import BeautifulSoup

from .const import commands, port, urls


class Helper:
    """Helper class to comuunicate with the devices."""

    @staticmethod
    def sendcommand(host, command):
        """Send command to the Sky Q box"""
        code = commands[command]
        cmd1 = int(math.floor(224 + (code / 16)))
        cmd2 = int(code % 16)
        command1 = array.array('B', [4, 1, 0, 0, 0, 0, cmd1, cmd2]).tostring()
        command2 = array.array('B', [4, 0, 0, 0, 0, 0, cmd1, cmd2]).tostring()

        s = socket.socket()
        s.connect((host, port))  # connect to Sky TV box
        reply = s.recv(12)  # Receive handshake
        s.send(reply)  # send handshake
        reply = s.recv(2)  # Receive 2 bytes
        s.send(reply)  # send 1 byte
        reply = s.recv(24)  # Receive 24 bytes

        s.send(command1)  # send command bytes part 1
        s.send(command2)  # send command bytes part 2

        s.close()  # close connection

    @staticmethod
    def request(request_type, response_type, target, request_url):
        """JSON Call to Sky Box"""
        json_url = request_url
        response = None
        if target != "N/A":
            json_url = urls["start"] + str(target) + str(request_url)

        if request_type == 'POST':
            response = requests.post(json_url, data=None, headers=None)
        elif request_type == 'GET':
            response = requests.get(json_url, data=None, headers=None)

        if response_type == 'JSON':
            json_data = response.json()
            return json_data
        else:
            return response

    @staticmethod
    def soap_request(command, ctrl_url):
        """Make a Soap request"""
        xml = """<?xml version="1.0" encoding="utf-8"?>
                <s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">
                <s:Body>
                <u:{} xmlns:u="urn:schemas-nds-com:service:SkyPlay:2">
                <InstanceID>0</InstanceID>
                </u:{}>
                </s:Body>
                </s:Envelope>""".format(command, command)

        headers = {
            "Content-Length": "{}".format(len(xml)),
            "SOAPACTION":
                "\"urn:schemas-nds-com:service:SkyPlay:2#{}\"".format(command),
            "Content-Type": "text/xml; char-set=utf-8"}

        xml_response = requests.post(url=ctrl_url, data=xml, headers=headers)
        return BeautifulSoup(xml_response.text, "lxml-xml")

    @staticmethod
    def epochtime(date_time, pattern, action):
        """ date/time conversion to epoch"""
        if action == 'to_epoch':
            pattern = '%d.%m.%Y %H:%M:%S'
            epochtime = int(time.mktime(
                time.strptime(str(date_time), pattern)))
            return epochtime
        elif action == 'from_epoch':
            date = datetime.fromtimestamp(int(date_time)).strftime(pattern)
            return date

    @staticmethod
    def get_sec(epoch):
        """Convert epoch time into total seconds."""
        clock = time.strftime('%H:%M:%S', time.localtime(epoch))
        h, m, s = clock.split(':')
        return int(h) * 3600 + int(m) * 60 + int(s)

    @staticmethod
    def xml(request):
        """Clean the xml response."""
        return BeautifulSoup(request.text, 'lxml-xml')



urls = {
    "start": "http://",
    "status": ":9006/as/system/status",
    "info_url": ":9006/as/system/information",
    "rec_list": ":9006/as/pvr?limit=1000&offset=0",
    "rec_info": ":9006/as/pvr/details/{0}",
    "chn_list": "http://awk.epgsky.com/hawk/linear/schedule/{0}/{1}",
    "channel_epg": ":9006/as/services",
    "channel_info": "9006/as/service/details/{0}",
    "metadata_url": "https://images.metadata.sky.com/pd-image/{0}/16-9?sid={1}",
    'xml_base': 'https://www.xmltv.co.uk'
}

commands = {
    "power": 0,
    "select": 1,
    "backup": 2,
    "dismiss": 2,
    "channelup": 6,
    "channeldown": 7,
    "interactive": 8,
    "sidebar": 8,
    "help": 9,
    "services": 10,
    "search": 10,
    "tvguide": 11,
    "home": 11,
    "i": 14,
    "text": 15,
    "up": 16,
    "down": 17,
    "left": 18,
    "right": 19,
    "red": 32,
    "green": 33,
    "yellow": 34,
    "blue": 35,
    "0": 48,
    "1": 49,
    "2": 50,
    "3": 51,
    "4": 52,
    "5": 53,
    "6": 54,
    "7": 55,
    "8": 56,
    "9": 57,
    "play": 64,
    "pause": 65,
    "stop": 66,
    "record": 67,
    "fastforward": 69,
    "rewind": 71,
    "boxoffice": 240,
    "sky": 241
}
dict_channel_list = None
port = 49160,
search_timeout = 5



import re
import socket
from typing import Dict

from pyskycontrol import Helper, SkyDevice


class Discover:
    """Discover Sky TV devices on the network."""

    def boxes(
        timeout: int = 30
    ) -> Dict[str, SkyDevice]:
        """
        Search for Sky Q Boxes on local network.

        :rtype: dict
        :return: Array of json objects {"ip", "port", "sys_info"}
        """
        device = {}
        devices = {}

        # UDP message.
        msg = \
            'M-SEARCH * HTTP/1.1\r\n' \
            'HOST:239.255.255.250:1900\r\n' \
            'ST:upnp:rootdevice\r\n' \
            'MX:2\r\n' \
            'MAN:"ssdp:discover"\r\n' \
            '\r\n'

        # Set up UDP socket.
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,
                          socket.IPPROTO_UDP)
        s.settimeout(timeout)
        s.sendto(msg.encode(), ('239.255.255.250', 1900))

        try:
            while True:
                data, addr = s.recvfrom(65507)
                data_text = data.decode("utf-8")
                url = re.search("(?P<url>https?://[^\s]+)",
                                data_text).group("url")
                if "Sky" in data_text:
                    if addr[0] not in device:
                        device[addr[0]] = []
                    device[addr[0]].append(url)
        except socket.timeout:
            pass

        compl = []
        for IP in device:
            for url in device[IP]:
                if IP in compl:
                    break
                else:
                    try:
                        headers = {
                            'USER-AGENT': 'SKY_skyplus',
                            'CONTENT-TYPE': 'text/xml; charset="utf-8"'
                        }
                        request = Helper.request.get(url=url, data=None,
                                                     headers=headers)
                        response = Helper.xml(request)
                        base_url = response.find(
                            'URLBase').string[:-1]
                        for node in response.find_all('service'):
                            service = node.serviceType.string
                            ctrl_url = base_url + node.controlURL.string
                            scpd_url = base_url + node.SCPDURL.string

                            if "SkyPlay:2" in service and \
                                    "player_avt.xml" in scpd_url:
                                result = self.request(request_type="GET",
                                                      response_type='JSON',
                                                      target=IP,
                                                      request_url=self.urls[
                                                          "info_url"])
                                name = result["btID"]
                                devices[IP] = {"IP": IP,
                                               "PORT": self.port,
                                               "NAME": name,
                                               "XML_URL": url,
                                               "CONTROL_URL": ctrl_url,
                                               "SCPD_URL": scpd_url,
                                               "SERVICE_TYPE": service}
                                compl.append(IP)
                    except:
                        pass

        return devices



from setuptools import setup, find_packages

setup(
    name='pyskycontrol',
    version='0.1.30',
    description='A Python library to interface with Sky',
    long_description="A Python library to interface with the Sky",
    url='https://Ktech6@bitbucket.org/Ktech6/pyskycontrol.git',

    author='ktech6',
    author_email='ktech6@outlook.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        #        'Topic :: Software Development :: API',
        'License :: OSI Approved :: MIT License',
        #        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    python_requires='>=3.5.*',
    keywords='Sky Library',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    #    packages=["pyhiveapi"],
    #    install_requires=[],

    install_requires=['requests', 'beautifulsoup4', 'lxml']
)
